<?php
/*
Plugin Name: LDAP Authentication Plug-in
Plugin URI: http://wpmuldap.frozenpc.net
Description: A plugin to override the core Wordpress MU authentication method so as to use an LDAP server for authentication.  
Version: 2.8.1
Author: Alistair Young (http://www.weblogs.uhi.ac.uk/sm00ay/), 
	Patrick Cavit (http://patcavit.com), 
	Hugo Salgado (http://hugo.vulcano.cl), 
	Alex Barker (http://www.callutheran.edu), 
	Dexter Arver, 
	Sean Wedig (http://www.thecodelife.net), 
	Aaron Axelsen (http://www.frozenpc.net)
*/

// Includes
require_once("ldap/lib/ldap_ro.php");

// methods for supporting site-admin configuration of the plugin
require_once("ldap/lib/wpmu_ldap_admin.functions.php");
require_once("ldap/lib/wpmu_ldap_adduser.functions.php");

add_action('admin_head', 'ldapShowOptionsCss');
add_action('admin_menu', 'ldap_addmenu');
add_action('admin_menu', 'ldap_addmenuuser');
add_action('admin_menu', 'wpmuRemoveAddNewMenu');

// perform these filters, actions, and WP function overrides only if LDAP-
//  authentication is enabled; this is to cut down on parsing of this code when
//  it doesn't apply 
if (get_site_option("ldapAuth")) { 

	// Add radio buttons for switching individual users between LDAP accounts and non-LDAP accounts
	add_action('edit_user_profile', 'wpmuUserFormLdapOption');
	add_action('edit_user_profile_update', 'wpmuUserFormLdapOptionUpdate');

	// *** End Admin Config Functions *** //

	// *** Begin User Auth Functions *** //
	// disable public signup if configured to do so  
	if (get_site_option('ldapDisableSignup')) {
		add_action('signup_header', 'wpmuLdapDisableSignup');
		add_action('login_head', 'wpmuLdapDisableSignupMessage');
	}

	if (get_site_option('ldapSSOEnabled')) add_action('wp_authenticate', 'ldapSSOAuthenticate', 10, 2);

	// only include them if it's active, so as to cut down on continual parsing of the code 
	require_once("ldap/lib/wpmu_ldap.functions.php");

	// disable only for ldap accounts
	add_filter('show_password_fields', 'wpmuLdapDisableLdapPassword');

	// disable default add user box	
	add_filter('show_adduser_fields', 'wpmuLdapDisableShowUser');

	function wp_authenticate($username,$password) {
		$isSSO = ($username == '[LDAP_SSO]');
		if ($isSSO) {
			/*
			AUTH_USER: The name of the user as it is derived from the authorization
			header sent by the client, before the user name is mapped to a Windows
			account. This variable is no different from REMOTE_USER. If you have an
			authentication filter installed on your Web server that maps incoming users
			to accounts, use LOGON_USER to view the mapped user name.

			LOGON_USER: The Windows account that the user is impersonating while
			connected to your Web server. Use REMOTE_USER or AUTH_USER to view the raw
			user name that is contained in the request header. The only time LOGON_USER
			holds a different value than these other variables is if you have an
			authentication filter installed.

			REMOTE_USER: The name of the user as it is derived from the authorization
			header sent by the client, before the user name is mapped to a Windows
			account. If you have an authentication filter installed on your Web server
			that maps incoming users to accounts, use LOGON_USER to view the mapped user
			name.
			*/
			$sso_user = '';
			if (!empty($_SERVER['LOGON_USER']))
				$sso_user = $_SERVER['LOGON_USER'];
			elseif (!empty($_SERVER['REMOTE_USER']))
				$sso_user = $_SERVER['REMOTE_USER'];
			elseif(!empty($_SERVER['AUTH_USER']))
				$sso_user = $_SERVER['AUTH_USER'];

			// strip user account domain
			if (strpos($sso_user, '\\\\') !== FALSE) {
				$sso_user = substr($sso_user, strpos($sso_user, '\\\\') + 2);
			}
			if (!empty($sso_user)) {
				$username = $sso_user;
				$password = wp_generate_password(); //create a random password for the local user
			} else {
				$isSSO = false;
			}
		}

		// make sure we always use lowercase usernames
		$username = strtolower($username);
		$username = sanitize_user($username);		

		// Is Ldap enabled?
		$ldapAuth = get_site_option("ldapAuth");

		// setup ldap string
		$ldapString = wpmuSetupLdapOptions();

		// try finding a WP account for this user name
		$user = get_userdatabylogin($username);

		// if username exists, lets check and see if they are already logged in
		if ($user != false) {
			$ldap_login = get_usermeta($user->ID,'ldap_login');

			//Setup redirection to users home directory.
			if (!strpos($_REQUEST['redirect_to'], $username)) {
				$_REQUEST['redirect_to'] = $username . "/" . $_REQUEST['redirect_to'];
			}

			// Try for local login
			if ($ldap_login != true || $ldap_login != 'true') {
				if ( !$user || ($user->user_login != $username) ) {
					do_action( 'wp_login_failed', $username );
					return new WP_Error('invalid_username', __('<strong>ERROR</strong>: Invalid username.'));
				}

				if ( !wp_check_password($password, $user->user_pass, $user->ID) ) {
					do_action( 'wp_login_failed', $username );
					return new WP_Error('incorrect_password', __('<strong>ERROR</strong>: Incorrect password.'));
				}
	                        wpmuUpdateBlogAccess($user->ID);

				return new WP_User($user->ID);
			}
		}

		if ($isSSO) {
			// search for user and retrieve user data
			$ldapString = wpmuSetupLdapOptions();
			$userDataArray = null;
			$result = wpmuLdapSearch($ldapString,$username,$userDataArray);
			$ldapauthresult = array('result' => $result, 'userdata' => $userDataArray);
		} else {
			// Authenticate via LDAP, potentially creating a WP user
			$ldapauthresult = wpmuLdapAuthenticate($ldapString, $username, $password);
		}

		if ($ldapauthresult['result']) {
			return wpmuLdapProcess($user, $username, $password, $ldapauthresult['userdata']);
		} elseif ($isSSO) {
			return new WP_Error('sso_failed', sprintf(__('Single Sign-On as user <em>%s</em> failed. Please login using the form below.'),$username));
		} else {
			return $ldapauthresult['errors'];
		}
	}
}
?>
